@@ -33,9 +33,11 @@
 #include "config.h"
 #include "filemap.h"
 #include "logging.h"
+#include "util.h"
 
 typedef struct job
 {
+    char errbuf[CURL_ERROR_SIZE];
     download_cb_t callback;
     void *opaque;
     CURL *request;
@@ -136,6 +138,10 @@ static CURL * setup_curl_easy(download_t *dl,
     if (err_setopt)
         goto fail;
 
+    err_setopt = curl_easy_setopt(request, CURLOPT_ERRORBUFFER, job->errbuf);
+    if (err_setopt)
+        goto fail;
+
     err_setopt = curl_easy_setopt(request,
         CURLOPT_USERAGENT,
         PACKAGE_NAME "/" PACKAGE_VERSION
@@ -144,7 +150,7 @@ static CURL * setup_curl_easy(download_t *dl,
         goto fail;
 
     err_setopt = curl_easy_setopt(request, CURLOPT_PROTOCOLS,
-        CURLPROTO_GOPHER | CURLPROTO_HTTP | CURLPROTO_HTTPS
+        CURLPROTO_FILE | CURLPROTO_GOPHER | CURLPROTO_HTTP | CURLPROTO_HTTPS
     );
     if (err_setopt)
         goto fail;
@@ -165,6 +171,10 @@ static CURL * setup_curl_easy(download_t *dl,
     if (err_setopt)
         goto fail;
 
+    err_setopt = curl_easy_setopt(request, CURLOPT_DEFAULT_PROTOCOL, "https");
+    if (err_setopt)
+        goto fail;
+
     err_setopt = curl_easy_setopt(request, CURLOPT_PRIVATE, job);
     if (err_setopt)
         goto fail;
@@ -273,7 +283,7 @@ fail:
     return -1;
 }
 
-static int request_failed(const job_t *job)
+static download_status_t request_failed(const job_t *job)
 {
     CURLcode err;
     long protocol, code;
@@ -283,45 +293,67 @@ static int request_failed(const job_t *job)
         warnx("curl_easy_getinfo CURLINFO_RESPONSE_CODE: %s",
             curl_easy_strerror(err)
         );
-        return -1;
+        return dl_sys_failure;
     }
 
     switch (protocol)
     {
-        // case CURLPROTO_FILE: - TODO should we use libcurl for local
-        //                        files too?
-        case CURLPROTO_GOPHER:
-            /* Gopher is not very good at error handling.  There's no
-             * generalized way of detecting errors.  E.g. gophernicus will
-             * report erros with some user-friendly text.  Assuming
-             * there's no error.
-             */
-            return 0;
+    case CURLPROTO_FILE:
+    case CURLPROTO_GOPHER:
+        /* Gopher is not very good at error handling.  There's no
+         * generalized way of detecting errors.  E.g. gophernicus will
+         * report erros with some user-friendly text.  Assuming
+         * there's no error.
+         */
+        break;
 
-        case CURLPROTO_HTTP:
-        case CURLPROTO_HTTPS:
-            err = curl_easy_getinfo(
-                job->request,
-                CURLINFO_RESPONSE_CODE,
-                &code
+    case CURLPROTO_HTTP:
+    case CURLPROTO_HTTPS:
+        err = curl_easy_getinfo(
+            job->request,
+            CURLINFO_RESPONSE_CODE,
+            &code
+        );
+        if (err) {
+            warnx("curl_easy_getinfo CURLINFO_RESPONSE_CODE: %s",
+                curl_easy_strerror(err)
             );
-            if (err) {
-                warnx("curl_easy_getinfo CURLINFO_RESPONSE_CODE: %s",
-                    curl_easy_strerror(err)
-                );
-                return -1;
-            }
-            return code < 200 || code >= 300;
+            return dl_sys_failure;
+        }
+        return (code < 200 || code >= 300) ? dl_fetch_failure : dl_complete;
     }
 
     /* Not knowing any better, we assume a failure if the file was never
      * assigned so far by a write operation. */
-    return job->output_file != -1;
+    return job->output_file == -1 ? dl_fetch_failure : dl_complete;
 }
 
-static int report_failure(job_t *job, download_status_t dls)
+static int report_failure(job_t *job, download_status_t type)
 {
-    return job->callback(job->opaque, dls, NULL);
+    const char *errmsg = NULL;
+
+    switch (type) {
+    case dl_complete:
+        panic("report_failure called with dl_complete");
+    case dl_aborted:
+        errmsg = "aborted";
+        break;
+    case dl_fetch_failure:
+        /* errbuf might be empty, e.g. when using file:/// and the file does
+         * not exist.  Using a generic message, it is better than nothing. */
+        errmsg = job->errbuf[0] ? job->errbuf : "not found";
+        break;
+    case dl_sys_failure:
+        /* More specific info is already reported via warn(3) or warnx(3). */
+        errmsg = "system failure";
+    }
+
+    return job->callback(
+        job->opaque,
+        type,
+        errmsg,
+        NULL
+    );
 }
 
 static int deliver(job_t *job)
@@ -338,7 +370,12 @@ static int deliver(job_t *job)
         return report_failure(job, dl_sys_failure);
     job->output_file = -1; /* on success, file ownership is moved */
 
-    e = job->callback(job->opaque, dl_complete, &filemap);
+    e = job->callback(
+        job->opaque,
+        dl_complete,
+        "success",
+        &filemap
+    );
     filemap_unload(&filemap);
     return e;
 }
@@ -351,46 +388,42 @@ static int flush_ready(download_t *dl)
     while (msg = curl_multi_info_read(dl->curlm, &msgq_ign), msg) {
         job_t *job;
         int cancel;
-        CURLcode err;
+        CURLcode e;
 
         if (msg->msg != CURLMSG_DONE)
             continue;
 
-        err = curl_easy_getinfo(msg->easy_handle, CURLINFO_PRIVATE, &job);
-        if (err) {
+        e = curl_easy_getinfo(msg->easy_handle, CURLINFO_PRIVATE, &job);
+        if (e) {
             /* This should not happen, but if it does, the job delivery
              * and deletion is going to be postponed until all downloads
              * are complete.
              */
             warnx("curl_easy_getinfo CURLINFO_PRIVATE: %s",
-                curl_easy_strerror(err)
+                curl_easy_strerror(e)
             );
             continue;
         }
 
-        if (msg->data.result == CURLE_OK) {
+        if (msg->data.result != CURLE_OK)
+            cancel = report_failure(job, dl_fetch_failure);
+        else
             switch (request_failed(job)) {
-            case -1:
+            case dl_sys_failure:
                 cancel = report_failure(job, dl_sys_failure);
                 break;
 
-            case 0:
+            case dl_complete:
                 cancel = deliver(job);
                 break;
 
-            case 1:
+            case dl_fetch_failure:
                 cancel = report_failure(job, dl_fetch_failure);
                 break;
 
-            default:
-                errx(EX_SOFTWARE, "unexpected request_failed value");
+            case dl_aborted:
+                panic("unexpected request_failed value");
             }
-        } else {
-            warnx("curl_multi_info_read: %s",
-                curl_easy_strerror(msg->data.result)
-            );
-            cancel = report_failure(job, dl_fetch_failure);
-        }
 
         job_delete(dl, job);
         if (cancel)
@@ -414,12 +447,12 @@ int download_perform(download_t *dl)
         }
 
         if (flush_ready(dl)) {
-            say("further transfers aborted");
+            notice("further transfers aborted");
             break;  /* canceled by delivery callback */
         }
 
         if (!still_running) {
-            say("no more transfers are running");
+            debug("no more running downloads");
             break;
         }
 
@@ -444,13 +477,13 @@ void download_del(download_t *dl)
 
         DL_FOREACH_SAFE(dl->jobs, job, tmp) {
             /* Deliver failure for all aborted downloads. */
-            job->callback(job->opaque, dl_aborted, NULL);
+            (void)report_failure(job, dl_aborted);
             job_delete(dl, job);
             ++interrupted;
         }
 
         if (interrupted)
-            say("%d download jobs were interrupted", interrupted);
+            notice("%d download jobs were interrupted", interrupted);
     }
 
     curl_multi_cleanup(dl->curlm);
