@@ -31,7 +31,7 @@ typedef enum {
     dl_sys_failure,
 } download_status_t;
 
-struct filemap_t;
+typedef struct filemap filemap_t;
 
 /* download_cb_t - download callback.
  *
@@ -41,7 +41,8 @@ struct filemap_t;
 typedef int (*download_cb_t)(
     void *opaque,
     download_status_t,
-    const struct filemap_t *filemap
+    const char *errmsg,
+    const filemap_t *filemap
 );
 
 download_t *download_new(bool global_init, unsigned maxjobs);
