@@ -2,6 +2,7 @@
 
 #include <err.h>
 #include <fcntl.h>
+#include <stdlib.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -20,6 +21,18 @@ static int filesize(int fd, size_t *s)
     return 0;
 }
 
+filemap_t *filemap_new(void)
+{
+    filemap_t *fmap;
+
+    fmap = malloc(sizeof(*fmap));
+    if (fmap)
+        *fmap = (filemap_t){};
+    else
+        warn("malloc");
+    return fmap;
+}
+
 int filemap_load_fd(filemap_t *fmap, int fd)
 {
     size_t size;
@@ -76,3 +89,12 @@ void filemap_unload(filemap_t *fmap)
     munmap(fmap->memory, fmap->size);
     *fmap = (filemap_t) {};
 }
+
+void filemap_del(filemap_t *fmap)
+{
+    if (!fmap)
+        return;
+
+    filemap_unload(fmap);
+    free(fmap);
+}
