@@ -19,6 +19,7 @@
  */
 
 #include "launcher.h"
+#include "util.h"
 
 #include <err.h>
 #include <errno.h>
@@ -96,7 +97,7 @@ static int child(const launcher_run_t *run, run_mode_t mode, int pipefd[2])
         if (open("/dev/null", O_RDONLY) == -1)    /* replacing closed stdin */
             err(EX_OSERR, "child open /dev/null failed");
     } else
-        err(EX_SOFTWARE, "unhandled case");
+        panic("unhandled case %d", mode);
 
     execvp(run->argv[0], (char **)run->argv);
     err(EX_OSERR, "faulty execvp(\"%s\", ...)", run->argv[0]);
