@@ -19,4 +19,4 @@
  */
 
 /* by law (!) the only global variable in the whole program */
-unsigned g_verbosity_level;
+unsigned g_verbosity_level = 1;
