@@ -34,6 +34,7 @@
 #include "outfmt_parser.h"
 #include "outfmt.h"
 #include "text.h"
+#include "util.h"
 
 enum {
     /* TODO: these assignments are arbitrary, and they
@@ -83,7 +84,7 @@ struct ofmt {
     ofmt_mode_t mode;
     void *opaque;
     void (*opaque_free)(void*);
-    radix_t resolvers_map;
+    radix_t *resolvers_map;
     ofmt_resolver_t pipe_resolver;
 
     struct ofp_setup ofp_setup;
@@ -99,7 +100,7 @@ struct ofmt {
     };
 };
 
-static int fail(ofmt_t ofmt, ofmt_fail_t why)
+static int fail(ofmt_t *ofmt, ofmt_fail_t why)
 {
     assert(why != ofmt_fail_bad_state);
     ofmt->error.reason = why;
@@ -108,7 +109,7 @@ static int fail(ofmt_t ofmt, ofmt_fail_t why)
     return -1;
 }
 
-static int get_resolver(ofmt_t ofmt,
+static int get_resolver(ofmt_t *ofmt,
                         const ofp_token_t *tok,
                         ofmt_resolver_t *res)
 {
@@ -136,7 +137,7 @@ static int get_resolver(ofmt_t ofmt,
     return n + 1;
 }
 
-static int print_save_pholder(ofmt_t ofmt, const ofp_token_t *tok)
+static int print_save_pholder(ofmt_t *ofmt, const ofp_token_t *tok)
 {
     struct print *print;
     ofmt_resolver_t resolver;
@@ -175,10 +176,13 @@ static size_t copy_escaped(char *dst, const char *src, size_t len)
 
         if (escaped) {
             escaped = false;
-            if (c == 'n')
-                c = '\n';
-            if (c == ':')   /* special \: interrupts a placeholder */
-                continue;
+            switch (c) {
+            case 'n': c = '\n'; break;
+            case 'r': c = '\r'; break;
+            case 't': c = '\t'; break;
+            case ':':
+                continue; /* \: is a "zero-width break". */
+            }
         } else if (c == '\\') {
             escaped = true;
             continue;
@@ -190,7 +194,7 @@ static size_t copy_escaped(char *dst, const char *src, size_t len)
     return j;
 }
 
-static int store(ofmt_t ofmt, const char *begin, size_t len)
+static int store(ofmt_t *ofmt, const char *begin, size_t len)
 {
     if (ofmt->buflen - ofmt->bufwrite < len) {
         /* it should be enough by construction, if it isn't it means the
@@ -206,7 +210,7 @@ static int store(ofmt_t ofmt, const char *begin, size_t len)
 
 static void record_fail(void *o, ofp_fail_t reason)
 {
-    ofmt_t ofmt = o;
+    ofmt_t *ofmt = o;
 
     /* we returned -1 during *_emit, so we already failed */
     if (reason == ofp_fail_emitting)
@@ -230,7 +234,7 @@ static inline struct subproc_segment *cur_segment(struct subproc *subproc)
     return &subproc->segments[subproc->segwrite - 1];
 }
 
-static int subproc_merge_verbatim(ofmt_t ofmt,
+static int subproc_merge_verbatim(ofmt_t *ofmt,
                                   struct subproc_segment *s,
                                   const ofp_token_t *tok)
 {
@@ -244,7 +248,7 @@ static int subproc_merge_verbatim(ofmt_t ofmt,
     return 0;
 }
 
-static int subproc_save_verbatim(ofmt_t ofmt, const ofp_token_t *tok)
+static int subproc_save_verbatim(ofmt_t *ofmt, const ofp_token_t *tok)
 {
     struct subproc_segment *s;
     int w;
@@ -279,7 +283,7 @@ static int subproc_save_verbatim(ofmt_t ofmt, const ofp_token_t *tok)
     return 0;
 }
 
-static int subproc_save_pholder(ofmt_t ofmt, const ofp_token_t *tok)
+static int subproc_save_pholder(ofmt_t *ofmt, const ofp_token_t *tok)
 {
     struct subproc_segment *s;
     ofmt_resolver_t resolver;
@@ -303,7 +307,7 @@ static int subproc_save_pholder(ofmt_t ofmt, const ofp_token_t *tok)
     return used;
 }
 
-static int subproc_save_whitespace(ofmt_t ofmt)
+static int subproc_save_whitespace(ofmt_t *ofmt)
 {
     struct subproc_segment *s;
 
@@ -331,7 +335,7 @@ static int subproc_save_whitespace(ofmt_t ofmt)
 
 static int subproc_emit(void *o, const ofp_token_t *tok)
 {
-    ofmt_t ofmt = o;
+    ofmt_t *ofmt = o;
 
     switch (tok->atom) {
     case ofp_atom_pholder:
@@ -347,7 +351,7 @@ static int subproc_emit(void *o, const ofp_token_t *tok)
     return 0;
 }
 
-static int print_save_verbatim(ofmt_t ofmt, const ofp_token_t *tok)
+static int print_save_verbatim(ofmt_t *ofmt, const ofp_token_t *tok)
 {
     int start;
     struct print *print;
@@ -370,7 +374,7 @@ static int print_save_verbatim(ofmt_t ofmt, const ofp_token_t *tok)
     return 0;
 }
 
-static int subproc_post_scan(ofmt_t ofmt)
+static int subproc_post_scan(ofmt_t *ofmt)
 {
     struct subproc *subproc;
     int arg_idx;
@@ -409,7 +413,7 @@ static int subproc_post_scan(ofmt_t ofmt)
 
 static int print_emit(void *o, const ofp_token_t *tok)
 {
-    ofmt_t ofmt = o;
+    ofmt_t *ofmt = o;
 
     switch (tok->atom) {
     case ofp_atom_pholder:
@@ -423,12 +427,12 @@ static int print_emit(void *o, const ofp_token_t *tok)
     return 0;
 }
 
-static int ensure_buffer(ofmt_t ofmt, size_t speclen)
+static int ensure_buffer(ofmt_t *ofmt, size_t speclen)
 {
     size_t needed;
     char *new_buffer;
 
-    /* ofmt_mode_subprocess:
+    /* ofmt_mode_exec:
      *  Fill up the buffer with nul-terminated chunks which will populate
      *  the execve(2) array.  Since each chunk is terminated by a space
      *  character we use (speclen + 1) characters: the +1 accounts for the
@@ -458,7 +462,7 @@ static int ensure_buffer(ofmt_t ofmt, size_t speclen)
     return 0;
 }
 
-static int print_eval(ofmt_t ofmt, const ofmt_evaluate_params_t *params)
+static int print_eval(ofmt_t *ofmt, const ofmt_evaluate_params_t *params)
 {
     struct print *print = &ofmt->print;
 
@@ -486,8 +490,7 @@ static int dry_run(const launcher_run_t *lch)
 {
     char buffer[60];
 
-    warnx("dry_run!");
-    warnx(" would execute: %s", text_join_argv(
+    warnx("dry run, would execute: %s", text_join_argv(
         buffer,
         sizeof(buffer),
         lch->argv
@@ -507,7 +510,7 @@ static int dry_run(const launcher_run_t *lch)
     return 0;   /* of course */
 }
 
-static int launch(ofmt_t ofmt, const launcher_run_t *lch)
+static int launch(ofmt_t *ofmt, const launcher_run_t *lch)
 {
     int status;
 
@@ -523,7 +526,7 @@ static int launch(ofmt_t ofmt, const launcher_run_t *lch)
     return -1;
 }
 
-static int subproc_eval(ofmt_t ofmt, const ofmt_evaluate_params_t *params)
+static int subproc_eval(ofmt_t *ofmt, const ofmt_evaluate_params_t *params)
 {
     struct subproc *subproc = &ofmt->subproc;
     char const **argv = subproc->argv;
@@ -562,7 +565,7 @@ static int subproc_eval(ofmt_t ofmt, const ofmt_evaluate_params_t *params)
             return fail(ofmt, ofmt_fail_bad_state);
 
         default:
-            err(EX_SOFTWARE, "invalid atom %d", s->atom);
+            panic("invalid atom %d", s->atom);
         }
 
         if (value_len + !!s->arg_boundary > sizeof evaluation_buffer - w)
@@ -576,7 +579,7 @@ static int subproc_eval(ofmt_t ofmt, const ofmt_evaluate_params_t *params)
     const char *input_stdin = NULL;
     if (subproc->pipe_enabled) {
         if (!ofmt->pipe_resolver)
-            errx(EX_SOFTWARE, "software bug: unregistered pipe resolver! Skipping");
+            panic("unregistered pipe resolver");
 
         input_stdin = ofmt->pipe_resolver(params->item, ofmt->opaque);
     }
@@ -593,10 +596,10 @@ static int subproc_eval(ofmt_t ofmt, const ofmt_evaluate_params_t *params)
         : launch(ofmt, &run);
 }
 
-ofmt_t ofmt_new(void)
+ofmt_t *ofmt_new(void)
 {
-    ofmt_t ofmt;
-    radix_t resolvers_map;
+    ofmt_t *ofmt;
+    radix_t *resolvers_map;
 
     ofmt = malloc(sizeof(struct ofmt));
     if (!ofmt) {
@@ -620,25 +623,25 @@ ofmt_t ofmt_new(void)
     return ofmt;
 }
 
-static void free_opaque(ofmt_t ofmt)
+static void free_opaque(ofmt_t *ofmt)
 {
     if (ofmt->opaque && ofmt->opaque_free)
         ofmt->opaque_free(ofmt->opaque);
 }
 
-void ofmt_set_opaque(ofmt_t ofmt, void *opaque, void (*opaque_free)(void *))
+void ofmt_set_opaque(ofmt_t *ofmt, void *opaque, void (*opaque_free)(void *))
 {
     free_opaque(ofmt);
     ofmt->opaque = opaque;
     ofmt->opaque_free = opaque_free;
 }
 
-void *ofmt_get_opaque(ofmt_t ofmt)
+void *ofmt_get_opaque(const ofmt_t *ofmt)
 {
     return ofmt->opaque;
 }
 
-int ofmt_set_resolver(ofmt_t ofmt, const char *key, ofmt_resolver_t resolver)
+int ofmt_set_resolver(ofmt_t *ofmt, const char *key, ofmt_resolver_t resolver)
 {
     if (radix_register(ofmt->resolvers_map, key, resolver) == -1)
         return fail(ofmt, errno == EINVAL ? ofmt_fail_bad_setup : ofmt_fail_system);
@@ -646,13 +649,13 @@ int ofmt_set_resolver(ofmt_t ofmt, const char *key, ofmt_resolver_t resolver)
     return 0;
 }
 
-int ofmt_set_pipe_resolver(ofmt_t ofmt, ofmt_resolver_t resolver)
+int ofmt_set_pipe_resolver(ofmt_t *ofmt, ofmt_resolver_t resolver)
 {
     ofmt->pipe_resolver = resolver;
     return 0;
 }
 
-static void reset(ofmt_t ofmt, ofmt_mode_t mode)
+static void reset(ofmt_t *ofmt, ofmt_mode_t mode)
 {
     ofmt->error = (ofmt_err_t){};
     ofmt->bufwrite = 0;
@@ -667,17 +670,17 @@ static void reset(ofmt_t ofmt, ofmt_mode_t mode)
     case ofmt_mode_pipe:
         ofmt->subproc.pipe_enabled = true;
         /* fallthrough */
-    case ofmt_mode_subprocess:
+    case ofmt_mode_exec:
         ofmt->subproc.segwrite = 0;
         ofmt->ofp_setup.emit = subproc_emit;
-        ofmt->mode = ofmt_mode_subprocess;
+        ofmt->mode = ofmt_mode_exec;
         break;
     default:
         abort();
     }
 }
 
-int ofmt_compile(ofmt_t ofmt, ofmt_mode_t mode,
+int ofmt_compile(ofmt_t *ofmt, ofmt_mode_t mode,
                  const char *spec, size_t speclen)
 {
     reset(ofmt, mode);
@@ -690,7 +693,7 @@ int ofmt_compile(ofmt_t ofmt, ofmt_mode_t mode,
 
     switch (mode) {
         case ofmt_mode_pipe:
-        case ofmt_mode_subprocess:
+        case ofmt_mode_exec:
             if (subproc_post_scan(ofmt) == -1)
                 return -1;
         case ofmt_mode_print:
@@ -699,10 +702,10 @@ int ofmt_compile(ofmt_t ofmt, ofmt_mode_t mode,
     return 0;
 }
 
-int ofmt_evaluate(ofmt_t ofmt, const ofmt_evaluate_params_t *params)
+int ofmt_evaluate(ofmt_t *ofmt, const ofmt_evaluate_params_t *params)
 {
     switch (ofmt->mode) {
-    case ofmt_mode_subprocess:
+    case ofmt_mode_exec:
         return subproc_eval(ofmt, params);
     case ofmt_mode_print:
         return print_eval(ofmt, params);
@@ -730,12 +733,12 @@ const char * ofmt_fail_str(ofmt_fail_t f)
     return repr[f];
 }
 
-const ofmt_err_t * ofmt_get_error(ofmt_t ofmt)
+const ofmt_err_t * ofmt_get_error(const ofmt_t *ofmt)
 {
     return &ofmt->error;
 }
 
-void ofmt_print_error(ofmt_t ofmt)
+void ofmt_print_error(const ofmt_t *ofmt)
 {
     switch (ofmt->error.reason) {
     case ofmt_fail_parse:
@@ -762,13 +765,13 @@ void ofmt_print_error(ofmt_t ofmt)
     }
 }
 
-void ofmt_del(ofmt_t ofmt)
+void ofmt_del(ofmt_t *ofmt)
 {
     if (!ofmt)
         return;
 
     int errno_s = errno;
-    radix_free(ofmt->resolvers_map);
+    radix_del(ofmt->resolvers_map);
     free_opaque(ofmt);
     free(ofmt->buffer);
     free(ofmt);
