@@ -27,24 +27,24 @@
 #include "outfmt_parser.h"
 #include "radix.h"
 
-typedef struct ofmt * ofmt_t;
+typedef struct ofmt ofmt_t;
 
-ofmt_t ofmt_new(void);
+ofmt_t * ofmt_new(void);
 
-void ofmt_set_opaque(ofmt_t, void *opaque, void (*opaque_free)(void *));
-void *ofmt_get_opaque(ofmt_t);
+void ofmt_set_opaque(ofmt_t *, void *opaque, void (*opaque_free)(void *));
+void *ofmt_get_opaque(const ofmt_t *);
 
 typedef const char * (*ofmt_resolver_t)(void *item, void *opaque);
-int ofmt_set_resolver(ofmt_t, const char *key, ofmt_resolver_t);
-int ofmt_set_pipe_resolver(ofmt_t, ofmt_resolver_t);
+int ofmt_set_resolver(ofmt_t *, const char *key, ofmt_resolver_t);
+int ofmt_set_pipe_resolver(ofmt_t *, ofmt_resolver_t);
 
 typedef enum {
+    ofmt_mode_exec,
     ofmt_mode_pipe,
     ofmt_mode_print,
-    ofmt_mode_subprocess,
 } ofmt_mode_t;
 
-int ofmt_compile(ofmt_t, ofmt_mode_t, const char *spec, size_t speclen);
+int ofmt_compile(ofmt_t *, ofmt_mode_t, const char *spec, size_t speclen);
 
 typedef struct {
     void *item;
@@ -53,7 +53,7 @@ typedef struct {
     bool dry_run;
 } ofmt_evaluate_params_t;
 
-int ofmt_evaluate(ofmt_t, const ofmt_evaluate_params_t *);
+int ofmt_evaluate(ofmt_t *, const ofmt_evaluate_params_t *);
 
 typedef enum {
     ofmt_fail_none = 0,
@@ -85,8 +85,8 @@ typedef struct {
     };
 } ofmt_err_t;
 
-const ofmt_err_t * ofmt_get_error(ofmt_t ofmt);
+const ofmt_err_t * ofmt_get_error(const ofmt_t *);
 
-void ofmt_print_error(ofmt_t ofmt);
+void ofmt_print_error(const ofmt_t *);
 
-void ofmt_del(ofmt_t);
+void ofmt_del(ofmt_t *);
