@@ -22,7 +22,7 @@
 
 #include "outfmt.h"
 
-int placeholders_setup(ofmt_t);
+int placeholders_setup(ofmt_t *);
 
 /* This struct collects additional information that gets copied in the
  * opaque structure within a ofmt_t object.  This information is
@@ -34,4 +34,4 @@ typedef struct {
     const char *feed_identifier;
 } placeholder_extra_t;
 
-void placeholders_set_extra(ofmt_t, const placeholder_extra_t *extra);
+void placeholders_set_extra(const ofmt_t *, const placeholder_extra_t *);
