@@ -22,10 +22,10 @@
 PLACEHOLDER("a",  author)
 PLACEHOLDER("am", author_email)
 PLACEHOLDER("au", author_uri)
-PLACEHOLDER("cm", comments)
-PLACEHOLDER("co", contributor)
+PLACEHOLDER("c",  contributor)
 PLACEHOLDER("cm", contributor_email)
 PLACEHOLDER("cu", contributor_uri)
+PLACEHOLDER("co", comments)
 PLACEHOLDER("cr", copyright)
 PLACEHOLDER("ct", copyright_type)
 PLACEHOLDER("d",  description)
@@ -44,6 +44,6 @@ PLACEHOLDER("t",  title)
 PLACEHOLDER("tt", title_type)
 
 /* These placeholders correspond to extra fields in placeholder_extra_t */
-PLACEHOLDER("n",  incremental_id)
-PLACEHOLDER("ft", feed_title)
 PLACEHOLDER("fi", feed_identifier)
+PLACEHOLDER("ft", feed_title)
+PLACEHOLDER("n",  incremental_id)
