@@ -35,12 +35,12 @@ struct radix {
     node_t *root;
 };
 
-int radix_lookup(radix_t r,
+int radix_lookup(const radix_t *r,
                  const char *path,
                  int pathlen,
                  void **out)
 {
-    node_t *c = r->root;
+    const node_t *c = r->root;
     int depth = 0;
     void *found = NULL;
 
@@ -69,7 +69,7 @@ exit:
     return depth;
 }
 
-int radix_register(radix_t r, const char *path, void *item)
+int radix_register(radix_t *r, const char *path, void *item)
 {
     node_t *c = r->root;
     node_t **back = &r->root;
@@ -118,9 +118,9 @@ static void node_free(node_t *n)
     free(n);
 }
 
-radix_t radix_new(void)
+radix_t *radix_new(void)
 {
-    radix_t r;
+    radix_t *r;
 
     r = malloc(sizeof(struct radix));
     if (r)
@@ -130,7 +130,7 @@ radix_t radix_new(void)
     return r;
 }
 
-void radix_free(radix_t r)
+void radix_del(radix_t *r)
 {
     if (!r)
         return;
