@@ -20,12 +20,12 @@
 
 #pragma once
 
-typedef struct radix * radix_t;
+typedef struct radix radix_t;
 
-radix_t radix_new(void);
+radix_t *radix_new(void);
 
-int radix_register(radix_t, const char *path, void *item);
+int radix_register(radix_t *, const char *path, void *item);
 
-int radix_lookup(radix_t, const char *path, int pathlen, void **found);
+int radix_lookup(const radix_t *, const char *path, int pathlen, void **found);
 
-void radix_free(radix_t);
+void radix_del(radix_t *);
