@@ -369,14 +369,14 @@ static int feed_load_magic(feed_t *feed)
      */
 
     if (feed->filemap.size < sizeof(magic_nr) + sizeof(revision_t)) {
-        whisper("feed [%s]: no magic number", feed_name(feed));
+        warn("feed [%s]: no magic number", feed_name(feed));
         goto unrecognized;
     }
 
     const uint8_t *bytes = feed->filemap.memory;
 
     if (strncmp(magic_nr, (const char *)bytes, sizeof(magic_nr)) != 0) {
-        whisper("feed [%s]: invalid magic number", feed_name(feed));
+        warn("feed [%s]: invalid magic number", feed_name(feed));
         goto unrecognized;
     }
     bytes += sizeof(magic_nr);
@@ -386,8 +386,8 @@ static int feed_load_magic(feed_t *feed)
 
     if (rev->major != magic_revision.major) {
         /* note - proper versioning is not supported yet */
-        whisper("feed [%s]: unmatched magic version (%d)",
-                feed_name(feed), rev->major);
+        warn("feed [%s]: unmatched magic version (%d)",
+             feed_name(feed), rev->major);
         feed->file_status = fs_incompat;
         return 0;
     }
@@ -456,9 +456,9 @@ static int feed_load(feed_t *feed, int fd)
                 feed->flags[i] = content.bytes[i];
 
             if (fi_flags_length < content.len) {
-                whisper("feed [%s]: ignoring %d flags",
-                        feed_name(feed),
-                        content.len - fi_flags_length);
+                warn("feed [%s]: ignoring %d flags",
+                     feed_name(feed),
+                     content.len - fi_flags_length);
             }
             break;
 
@@ -479,8 +479,8 @@ static int feed_load(feed_t *feed, int fd)
             warnx("feed [%s]: reports entry of kind ek_none??",
                   feed_name(feed));
         default:
-            whisper("feed [%s]: ignoring entry kind %d",
-                    feed_name(feed), ek);
+            warn("feed [%s]: ignoring entry kind %d",
+                 feed_name(feed), ek);
             break;
         }
 
@@ -928,6 +928,11 @@ const feed_t * savefile_citer_feeds(const savefile_t *save, void **aux)
     return savefile_iter_feeds((savefile_t *)save, aux);
 }
 
+int savefile_get_dirfd(const savefile_t *save)
+{
+    return save->dirfd;
+}
+
 bool feed_exists(const feed_t *feed)
 {
     return feed->file_status != fs_noent;
@@ -997,6 +1002,11 @@ const char *feed_get_name(const feed_t *feed)
     return feed_name(feed);
 }
 
+const char *feed_get_file_path(const feed_t *feed)
+{
+    return feed_name(feed);
+}
+
 str_t feed_get_outfmt(const feed_t *feed)
 {
     return STR_ALIAS(&feed->outfmt);
