@@ -78,6 +78,8 @@ feed_t * savefile_iter_feeds(savefile_t *, void **aux);
  */
 const feed_t * savefile_citer_feeds(const savefile_t *, void **aux);
 
+int savefile_get_dirfd(const savefile_t *);
+
 void savefile_free(savefile_t *);
 
 bool feed_exists(const feed_t *);
@@ -99,6 +101,7 @@ int feed_set_subproc_chdir(feed_t *, str_t *path);
 void feed_mark_deleted(feed_t *);
 
 const char *feed_get_name(const feed_t *);
+const char *feed_get_file_path(const feed_t *);
 
 /* getters, returned strings never have ownership */
 str_t feed_get_provided_url(const feed_t *);
