@@ -59,7 +59,7 @@ static int vardir(bool create)
     path = "/var/lib/crossbow";
 
 chosen:
-    whisper("configuration in \"%s\"", path);
+    notice("Old persist directory: %s", path);
     return open_dir(path, create);
 }
 
@@ -82,4 +82,3 @@ savefile_t *open_savefile(bool create)
 
     return savefile;
 }
-
