@@ -27,14 +27,4 @@ typedef struct util_ctx *util_ctx_t;
 util_ctx_t util_test_setup(const char *test_name);
 void util_test_teardown(util_ctx_t);
 
-int util_open(util_ctx_t, const char *path, int flags, mode_t mode);
-int util_mkdir(util_ctx_t ctx, const char *path, mode_t mode);
-
-struct util_rss_item {
-    const char *guid;
-    const char *link;
-};
-
-int util_gen_rss(util_ctx_t, const char *path, const struct util_rss_item *);
-
 int util_get_tempdir(util_ctx_t);
